@@ -23,7 +23,6 @@ import java.lang.management.MemoryUsage;
 import java.lang.reflect.Method;
 import java.text.MessageFormat;
 import java.util.Date;
-import java.util.Enumeration;
 import java.util.Iterator;
 import java.util.Set;
 import java.util.SortedMap;
@@ -47,12 +46,9 @@ import org.apache.tomcat.util.security.Escape;
  */
 public class StatusTransformer {
 
-
     // --------------------------------------------------------- Public Methods
 
-
-    public static void setContentType(HttpServletResponse response,
-                                      int mode) {
+    public static void setContentType(HttpServletResponse response, int mode) {
         if (mode == 0){
             response.setContentType("text/html;charset="+Constants.CHARSET);
         } else if (mode == 1){
@@ -67,15 +63,12 @@ public class StatusTransformer {
      * @param writer the PrintWriter to use
      * @param args Path prefix for URLs
      * @param mode - 0 = HTML header, 1 = XML declaration
-     *
      */
     public static void writeHeader(PrintWriter writer, Object[] args, int mode) {
-        if (mode == 0){
+        if (mode == 0) {
             // HTML Header Section
-            writer.print(MessageFormat.format(
-                Constants.HTML_HEADER_SECTION, args
-            ));
-        } else if (mode == 1){
+            writer.print(MessageFormat.format(Constants.HTML_HEADER_SECTION, args));
+        } else if (mode == 1) {
             writer.write(Constants.XML_DECLARATION);
             writer.print(MessageFormat.format
                      (Constants.XML_STYLE, args));
@@ -94,8 +87,7 @@ public class StatusTransformer {
      */
     public static void writeBody(PrintWriter writer, Object[] args, int mode) {
         if (mode == 0){
-            writer.print(MessageFormat.format
-                         (Constants.BODY_HEADER_SECTION, args));
+            writer.print(MessageFormat.format(Constants.BODY_HEADER_SECTION, args));
         }
     }
 
@@ -107,26 +99,22 @@ public class StatusTransformer {
      * @param args What to write
      * @param mode 0 means write
      */
-    public static void writeManager(PrintWriter writer, Object[] args,
-                                    int mode) {
-        if (mode == 0){
+    public static void writeManager(PrintWriter writer, Object[] args, int mode) {
+        if (mode == 0) {
             writer.print(MessageFormat.format(Constants.MANAGER_SECTION, args));
         }
     }
 
 
-    public static void writePageHeading(PrintWriter writer, Object[] args,
-                                        int mode) {
-        if (mode == 0){
-            writer.print(MessageFormat.format
-                         (Constants.SERVER_HEADER_SECTION, args));
+    public static void writePageHeading(PrintWriter writer, Object[] args, int mode) {
+        if (mode == 0) {
+            writer.print(MessageFormat.format(Constants.SERVER_HEADER_SECTION, args));
         }
     }
 
 
-    public static void writeServerInfo(PrintWriter writer, Object[] args,
-                                       int mode){
-        if (mode == 0){
+    public static void writeServerInfo(PrintWriter writer, Object[] args, int mode){
+        if (mode == 0) {
             writer.print(MessageFormat.format(Constants.SERVER_ROW_SECTION, args));
         }
     }
@@ -337,10 +325,8 @@ public class StatusTransformer {
 
             ObjectName grpName = null;
 
-            Enumeration<ObjectName> enumeration = globalRequestProcessors.elements();
-            // Find the HTTP/1.1 RequestGroupInfo - BZ 65404
-            while (enumeration.hasMoreElements()) {
-                ObjectName objectName = enumeration.nextElement();
+            for (ObjectName objectName : globalRequestProcessors) {
+                // Find the HTTP/1.1 RequestGroupInfo - BZ 65404
                 if (name.equals(objectName.getKeyProperty("name")) && objectName.getKeyProperty("Upgrade") == null) {
                     grpName = objectName;
                 }
@@ -352,13 +338,11 @@ public class StatusTransformer {
 
             writer.print( args[4] );
             writer.print(' ');
-            writer.print(formatTime(mBeanServer.getAttribute
-                                    (grpName, "maxTime"), false));
+            writer.print(formatTime(mBeanServer.getAttribute(grpName, "maxTime"), false));
             writer.print(' ');
             writer.print(args[5]);
             writer.print(' ');
-            writer.print(formatTime(mBeanServer.getAttribute
-                                    (grpName, "processingTime"), true));
+            writer.print(formatTime(mBeanServer.getAttribute(grpName, "processingTime"), true));
             writer.print(' ');
             writer.print(args[6]);
             writer.print(' ');
@@ -370,20 +354,16 @@ public class StatusTransformer {
             writer.print(' ');
             writer.print(args[8]);
             writer.print(' ');
-            writer.print(formatSize(mBeanServer.getAttribute
-                                    (grpName, "bytesReceived"), true));
+            writer.print(formatSize(mBeanServer.getAttribute(grpName, "bytesReceived"), true));
             writer.print(' ');
             writer.print(args[9]);
             writer.print(' ');
-            writer.print(formatSize(mBeanServer.getAttribute
-                                    (grpName, "bytesSent"), true));
+            writer.print(formatSize(mBeanServer.getAttribute(grpName, "bytesSent"), true));
             writer.print("</p>");
 
             writer.print("<table border=\"0\"><tr><th>"+ args[10] + "</th><th>" + args[11] + "</th><th>" + args[12] +"</th><th>" + args[13] +"</th><th>" + args[14] + "</th><th>" + args[15] + "</th><th>" + args[16] + "</th><th>" + args[17] + "</th></tr>");
 
-            enumeration = requestProcessors.elements();
-            while (enumeration.hasMoreElements()) {
-                ObjectName objectName = enumeration.nextElement();
+            for (ObjectName objectName : requestProcessors) {
                 if (name.equals(objectName.getKeyProperty("worker"))) {
                     writer.print("<tr>");
                     writeProcessorState(writer, objectName, mBeanServer, mode);
@@ -407,10 +387,8 @@ public class StatusTransformer {
 
             ObjectName grpName = null;
 
-            Enumeration<ObjectName> enumeration = globalRequestProcessors.elements();
-            // Find the HTTP/1.1 RequestGroupInfo - BZ 65404
-            while (enumeration.hasMoreElements()) {
-                ObjectName objectName = enumeration.nextElement();
+            for (ObjectName objectName : globalRequestProcessors) {
+                // Find the HTTP/1.1 RequestGroupInfo - BZ 65404
                 if (name.equals(objectName.getKeyProperty("name")) && objectName.getKeyProperty("Upgrade") == null) {
                     grpName = objectName;
                 }
@@ -428,9 +406,7 @@ public class StatusTransformer {
                 writer.write(" />");
 
                 writer.write("<workers>");
-                enumeration = requestProcessors.elements();
-                while (enumeration.hasMoreElements()) {
-                    ObjectName objectName = enumeration.nextElement();
+                for (ObjectName objectName : requestProcessors) {
                     if (name.equals(objectName.getKeyProperty("worker"))) {
                         writeProcessorState(writer, objectName, mBeanServer, mode);
                     }
@@ -440,7 +416,6 @@ public class StatusTransformer {
 
             writer.write("</connector>");
         }
-
     }
 
 
@@ -533,23 +508,23 @@ public class StatusTransformer {
                 }
                 writer.write("</td>");
                 writer.write("<td>");
-                writer.print(Escape.htmlElementContext(mBeanServer.getAttribute
+                writer.print(Escape.htmlElementContent(mBeanServer.getAttribute
                                     (pName, "remoteAddrForwarded")));
                 writer.write("</td>");
                 writer.write("<td>");
-                writer.print(Escape.htmlElementContext(mBeanServer.getAttribute
+                writer.print(Escape.htmlElementContent(mBeanServer.getAttribute
                                     (pName, "remoteAddr")));
                 writer.write("</td>");
                 writer.write("<td nowrap>");
-                writer.write(Escape.htmlElementContext(mBeanServer.getAttribute
+                writer.write(Escape.htmlElementContent(mBeanServer.getAttribute
                                     (pName, "virtualHost")));
                 writer.write("</td>");
                 writer.write("<td nowrap class=\"row-left\">");
                 if (showRequest) {
-                    writer.write(Escape.htmlElementContext(mBeanServer.getAttribute
+                    writer.write(Escape.htmlElementContent(mBeanServer.getAttribute
                                         (pName, "method")));
                     writer.write(' ');
-                    writer.write(Escape.htmlElementContext(mBeanServer.getAttribute
+                    writer.write(Escape.htmlElementContent(mBeanServer.getAttribute
                                         (pName, "currentUri")));
                     String queryString = (String) mBeanServer.getAttribute
                         (pName, "currentQueryString");
@@ -558,7 +533,7 @@ public class StatusTransformer {
                         writer.print(Escape.htmlElementContent(queryString));
                     }
                     writer.write(' ');
-                    writer.write(Escape.htmlElementContext(mBeanServer.getAttribute
+                    writer.write(Escape.htmlElementContent(mBeanServer.getAttribute
                                         (pName, "protocol")));
                 } else {
                     writer.write("?");
@@ -592,18 +567,18 @@ public class StatusTransformer {
                 }
                 writer.write("\"");
                 writer.write(" remoteAddr=\""
-                             + Escape.htmlElementContext(mBeanServer.getAttribute
+                             + Escape.htmlElementContent(mBeanServer.getAttribute
                                       (pName, "remoteAddr")) + "\"");
                 writer.write(" virtualHost=\""
-                             + Escape.htmlElementContext(mBeanServer.getAttribute
+                             + Escape.htmlElementContent(mBeanServer.getAttribute
                                       (pName, "virtualHost")) + "\"");
 
                 if (showRequest) {
                     writer.write(" method=\""
-                                 + Escape.htmlElementContext(mBeanServer.getAttribute
+                                 + Escape.htmlElementContent(mBeanServer.getAttribute
                                           (pName, "method")) + "\"");
                     writer.write(" currentUri=\""
-                                 + Escape.htmlElementContext(mBeanServer.getAttribute
+                                 + Escape.htmlElementContent(mBeanServer.getAttribute
                                           (pName, "currentUri")) + "\"");
 
                     String queryString = (String) mBeanServer.getAttribute
@@ -615,7 +590,7 @@ public class StatusTransformer {
                         writer.write(" currentQueryString=\"&#63;\"");
                     }
                     writer.write(" protocol=\""
-                                 + Escape.htmlElementContext(mBeanServer.getAttribute
+                                 + Escape.htmlElementContent(mBeanServer.getAttribute
                                           (pName, "protocol")) + "\"");
                 } else {
                     writer.write(" method=\"&#63;\"");
@@ -677,7 +652,7 @@ public class StatusTransformer {
                 }
 
                 writer.print("<a href=\"#" + (count++) + ".0\">");
-                writer.print(Escape.htmlElementContext(webModuleName));
+                writer.print(Escape.htmlElementContent(webModuleName));
                 writer.print("</a>");
                 if (iterator.hasNext()) {
                     writer.print("<br>");
@@ -759,7 +734,7 @@ public class StatusTransformer {
             }
 
             writer.print("<h1>");
-            writer.print(Escape.htmlElementContext(name));
+            writer.print(Escape.htmlElementContent(name));
             writer.print("</h1>");
             writer.print("</a>");
 
@@ -902,11 +877,11 @@ public class StatusTransformer {
                 mBeanServer.invoke(objectName, "findMappings", null, null);
 
             writer.print("<h2>");
-            writer.print(Escape.htmlElementContext(servletName));
+            writer.print(Escape.htmlElementContent(servletName));
             if ((mappings != null) && (mappings.length > 0)) {
                 writer.print(" [ ");
                 for (int i = 0; i < mappings.length; i++) {
-                    writer.print(Escape.htmlElementContext(mappings[i]));
+                    writer.print(Escape.htmlElementContent(mappings[i]));
                     if (i < mappings.length - 1) {
                         writer.print(" , ");
                     }
@@ -941,87 +916,10 @@ public class StatusTransformer {
 
 
     /**
-     * Filter the specified message string for characters that are sensitive
-     * in HTML.  This avoids potential attacks caused by including JavaScript
-     * codes in the request URL that is often reported in error messages.
-     *
-     * @param obj The message string to be filtered
-     * @return filtered HTML content
-     *
-     * @deprecated This method will be removed in Tomcat 9
-     */
-    @Deprecated
-    public static String filter(Object obj) {
-
-        if (obj == null) {
-            return ("?");
-        }
-        String message = obj.toString();
-
-        char content[] = new char[message.length()];
-        message.getChars(0, message.length(), content, 0);
-        StringBuilder result = new StringBuilder(content.length + 50);
-        for (int i = 0; i < content.length; i++) {
-            switch (content[i]) {
-            case '<':
-                result.append("&lt;");
-                break;
-            case '>':
-                result.append("&gt;");
-                break;
-            case '&':
-                result.append("&amp;");
-                break;
-            case '"':
-                result.append("&quot;");
-                break;
-            default:
-                result.append(content[i]);
-            }
-        }
-        return result.toString();
-
-    }
-
-
-    /**
-     * Escape the 5 entities defined by XML.
-     * @param s The message string to be filtered
-     * @return filtered XML content
-     *
-     * @deprecated This method will be removed in Tomcat 9
-     */
-    @Deprecated
-    public static String filterXml(String s) {
-        if (s == null) {
-            return "";
-        }
-        StringBuilder sb = new StringBuilder();
-        for (int i = 0; i < s.length(); i++) {
-            char c = s.charAt(i);
-            if (c == '<') {
-                sb.append("&lt;");
-            } else if (c == '>') {
-                sb.append("&gt;");
-            } else if (c == '\'') {
-                sb.append("&apos;");
-            } else if (c == '&') {
-                sb.append("&amp;");
-            } else if (c == '"') {
-                sb.append("&quot;");
-            } else {
-                sb.append(c);
-            }
-        }
-        return sb.toString();
-    }
-
-
-    /**
-     * Display the given size in bytes, either as KiB or MiB.
+     * Display the given size in bytes, either as KB or MB.
      *
      * @param obj The object to format
-     * @param mb true to display MiB, false for KiB
+     * @param mb true to display megabytes, false for kilobytes
      * @return formatted size
      */
     public static String formatSize(Object obj, boolean mb) {
@@ -1047,10 +945,10 @@ public class StatusTransformer {
             if (rest < 10) {
                 buff.append('0');
             }
-            buff.append(rest).append(" MiB");
+            buff.append(rest).append(" MB");
             return buff.toString();
         } else {
-            return ((bytes / 1024) + " KiB");
+            return ((bytes / 1024) + " KB");
         }
 
     }
