@@ -32,7 +32,7 @@ import org.apache.tomcat.util.res.StringManager;
 
 public class PerMessageDeflate implements Transformation {
 
-    private static final StringManager sm = StringManager.getManager(Constants.PACKAGE_NAME);
+    private static final StringManager sm = StringManager.getManager(PerMessageDeflate.class);
 
     private static final String SERVER_NO_CONTEXT_TAKEOVER = "server_no_context_takeover";
     private static final String CLIENT_NO_CONTEXT_TAKEOVER = "client_no_context_takeover";
@@ -40,7 +40,7 @@ public class PerMessageDeflate implements Transformation {
     private static final String CLIENT_MAX_WINDOW_BITS = "client_max_window_bits";
 
     private static final int RSV_BITMASK = 0b100;
-    private static final byte[] EOM_BYTES = new byte[] {0, 0, -1, -1};
+    private static final byte[] EOM_BYTES = new byte[] { 0, 0, -1, -1 };
 
     public static final String NAME = "permessage-deflate";
 
@@ -76,29 +76,25 @@ public class PerMessageDeflate implements Transformation {
                         serverContextTakeover = false;
                     } else {
                         // Duplicate definition
-                        throw new IllegalArgumentException(sm.getString(
-                                "perMessageDeflate.duplicateParameter",
-                                SERVER_NO_CONTEXT_TAKEOVER ));
+                        throw new IllegalArgumentException(
+                                sm.getString("perMessageDeflate.duplicateParameter", SERVER_NO_CONTEXT_TAKEOVER));
                     }
                 } else if (CLIENT_NO_CONTEXT_TAKEOVER.equals(param.getName())) {
                     if (clientContextTakeover) {
                         clientContextTakeover = false;
                     } else {
                         // Duplicate definition
-                        throw new IllegalArgumentException(sm.getString(
-                                "perMessageDeflate.duplicateParameter",
-                                CLIENT_NO_CONTEXT_TAKEOVER ));
+                        throw new IllegalArgumentException(
+                                sm.getString("perMessageDeflate.duplicateParameter", CLIENT_NO_CONTEXT_TAKEOVER));
                     }
                 } else if (SERVER_MAX_WINDOW_BITS.equals(param.getName())) {
                     if (serverMaxWindowBits == -1) {
                         serverMaxWindowBits = Integer.parseInt(param.getValue());
                         if (serverMaxWindowBits < 8 || serverMaxWindowBits > 15) {
-                            throw new IllegalArgumentException(sm.getString(
-                                    "perMessageDeflate.invalidWindowSize",
-                                    SERVER_MAX_WINDOW_BITS,
-                                    Integer.valueOf(serverMaxWindowBits)));
+                            throw new IllegalArgumentException(sm.getString("perMessageDeflate.invalidWindowSize",
+                                    SERVER_MAX_WINDOW_BITS, Integer.valueOf(serverMaxWindowBits)));
                         }
-                        // Java SE API (as of Java 8) does not expose the API to
+                        // Java SE API (as of Java 11) does not expose the API to
                         // control the Window size. It is effectively hard-coded
                         // to 15
                         if (isServer && serverMaxWindowBits != 15) {
@@ -111,28 +107,25 @@ public class PerMessageDeflate implements Transformation {
                         }
                     } else {
                         // Duplicate definition
-                        throw new IllegalArgumentException(sm.getString(
-                                "perMessageDeflate.duplicateParameter",
-                                SERVER_MAX_WINDOW_BITS ));
+                        throw new IllegalArgumentException(
+                                sm.getString("perMessageDeflate.duplicateParameter", SERVER_MAX_WINDOW_BITS));
                     }
                 } else if (CLIENT_MAX_WINDOW_BITS.equals(param.getName())) {
                     if (clientMaxWindowBits == -1) {
                         if (param.getValue() == null) {
                             // Hint to server that the client supports this
-                            // option. Java SE API (as of Java 8) does not
+                            // option. Java SE API (as of Java 11) does not
                             // expose the API to control the Window size. It is
                             // effectively hard-coded to 15
                             clientMaxWindowBits = 15;
                         } else {
                             clientMaxWindowBits = Integer.parseInt(param.getValue());
                             if (clientMaxWindowBits < 8 || clientMaxWindowBits > 15) {
-                                throw new IllegalArgumentException(sm.getString(
-                                        "perMessageDeflate.invalidWindowSize",
-                                        CLIENT_MAX_WINDOW_BITS,
-                                        Integer.valueOf(clientMaxWindowBits)));
+                                throw new IllegalArgumentException(sm.getString("perMessageDeflate.invalidWindowSize",
+                                        CLIENT_MAX_WINDOW_BITS, Integer.valueOf(clientMaxWindowBits)));
                             }
                         }
-                        // Java SE API (as of Java 8) does not expose the API to
+                        // Java SE API (as of Java 11) does not expose the API to
                         // control the Window size. It is effectively hard-coded
                         // to 15
                         if (!isServer && clientMaxWindowBits != 15) {
@@ -145,19 +138,18 @@ public class PerMessageDeflate implements Transformation {
                         }
                     } else {
                         // Duplicate definition
-                        throw new IllegalArgumentException(sm.getString(
-                                "perMessageDeflate.duplicateParameter",
-                                CLIENT_MAX_WINDOW_BITS ));
+                        throw new IllegalArgumentException(
+                                sm.getString("perMessageDeflate.duplicateParameter", CLIENT_MAX_WINDOW_BITS));
                     }
                 } else {
                     // Unknown parameter
-                    throw new IllegalArgumentException(sm.getString(
-                            "perMessageDeflate.unknownParameter", param.getName()));
+                    throw new IllegalArgumentException(
+                            sm.getString("perMessageDeflate.unknownParameter", param.getName()));
                 }
             }
             if (ok) {
-                return new PerMessageDeflate(serverContextTakeover, serverMaxWindowBits,
-                        clientContextTakeover, clientMaxWindowBits, isServer);
+                return new PerMessageDeflate(serverContextTakeover, serverMaxWindowBits, clientContextTakeover,
+                        clientMaxWindowBits, isServer);
             }
         }
         // Failed to negotiate agreeable terms
@@ -165,8 +157,8 @@ public class PerMessageDeflate implements Transformation {
     }
 
 
-    private PerMessageDeflate(boolean serverContextTakeover, int serverMaxWindowBits,
-            boolean clientContextTakeover, int clientMaxWindowBits, boolean isServer) {
+    private PerMessageDeflate(boolean serverContextTakeover, int serverMaxWindowBits, boolean clientContextTakeover,
+            int clientMaxWindowBits, boolean isServer) {
         this.serverContextTakeover = serverContextTakeover;
         this.serverMaxWindowBits = serverMaxWindowBits;
         this.clientContextTakeover = clientContextTakeover;
@@ -176,8 +168,7 @@ public class PerMessageDeflate implements Transformation {
 
 
     @Override
-    public TransformationResult getMoreData(byte opCode, boolean fin, int rsv, ByteBuffer dest)
-            throws IOException {
+    public TransformationResult getMoreData(byte opCode, boolean fin, int rsv, ByteBuffer dest) throws IOException {
         // Control frames are never compressed and may appear in the middle of
         // a WebSocket method. Pass them straight through.
         if (Util.isControl(opCode)) {
@@ -197,27 +188,25 @@ public class PerMessageDeflate implements Transformation {
         int written;
         boolean usedEomBytes = false;
 
-        while (dest.remaining() > 0) {
+        while (dest.remaining() > 0 || usedEomBytes) {
             // Space available in destination. Try and fill it.
             try {
-                written = inflater.inflate(
-                        dest.array(), dest.arrayOffset() + dest.position(), dest.remaining());
+                written = inflater.inflate(dest.array(), dest.arrayOffset() + dest.position(), dest.remaining());
             } catch (DataFormatException e) {
                 throw new IOException(sm.getString("perMessageDeflate.deflateFailed"), e);
+            } catch (NullPointerException e) {
+                throw new IOException(sm.getString("perMessageDeflate.alreadyClosed"), e);
             }
             dest.position(dest.position() + written);
 
-            if (inflater.needsInput() && !usedEomBytes ) {
+            if (inflater.needsInput() && !usedEomBytes) {
+                readBuffer.clear();
+                TransformationResult nextResult = next.getMoreData(opCode, fin, (rsv ^ RSV_BITMASK), readBuffer);
+                inflater.setInput(readBuffer.array(), readBuffer.arrayOffset(), readBuffer.position());
                 if (dest.hasRemaining()) {
-                    readBuffer.clear();
-                    TransformationResult nextResult =
-                            next.getMoreData(opCode, fin, (rsv ^ RSV_BITMASK), readBuffer);
-                    inflater.setInput(
-                            readBuffer.array(), readBuffer.arrayOffset(), readBuffer.position());
                     if (TransformationResult.UNDERFLOW.equals(nextResult)) {
                         return nextResult;
-                    } else if (TransformationResult.END_OF_FRAME.equals(nextResult) &&
-                            readBuffer.position() == 0) {
+                    } else if (TransformationResult.END_OF_FRAME.equals(nextResult) && readBuffer.position() == 0) {
                         if (fin) {
                             inflater.setInput(EOM_BYTES);
                             usedEomBytes = true;
@@ -225,11 +214,19 @@ public class PerMessageDeflate implements Transformation {
                             return TransformationResult.END_OF_FRAME;
                         }
                     }
+                } else if (readBuffer.position() > 0) {
+                    return TransformationResult.OVERFLOW;
+                } else if (fin) {
+                    inflater.setInput(EOM_BYTES);
+                    usedEomBytes = true;
                 }
             } else if (written == 0) {
-                if (fin && (isServer && !clientContextTakeover ||
-                        !isServer && !serverContextTakeover)) {
-                    inflater.reset();
+                if (fin && (isServer && !clientContextTakeover || !isServer && !serverContextTakeover)) {
+                    try {
+                        inflater.reset();
+                    } catch (NullPointerException e) {
+                        throw new IOException(sm.getString("perMessageDeflate.alreadyClosed"), e);
+                    }
                 }
                 return TransformationResult.END_OF_FRAME;
             }
@@ -242,7 +239,7 @@ public class PerMessageDeflate implements Transformation {
     @Override
     public boolean validateRsv(int rsv, byte opCode) {
         if (Util.isControl(opCode)) {
-            if ((rsv & RSV_BITMASK) > 0) {
+            if ((rsv & RSV_BITMASK) != 0) {
                 return false;
             } else {
                 if (next == null) {
@@ -253,7 +250,7 @@ public class PerMessageDeflate implements Transformation {
             }
         } else {
             int rsvNext = rsv;
-            if ((rsv & RSV_BITMASK) > 0) {
+            if ((rsv & RSV_BITMASK) != 0) {
                 rsvNext = rsv ^ RSV_BITMASK;
             }
             if (next == null) {
@@ -275,15 +272,13 @@ public class PerMessageDeflate implements Transformation {
             params.add(new WsExtensionParameter(SERVER_NO_CONTEXT_TAKEOVER, null));
         }
         if (serverMaxWindowBits != -1) {
-            params.add(new WsExtensionParameter(SERVER_MAX_WINDOW_BITS,
-                    Integer.toString(serverMaxWindowBits)));
+            params.add(new WsExtensionParameter(SERVER_MAX_WINDOW_BITS, Integer.toString(serverMaxWindowBits)));
         }
         if (!clientContextTakeover) {
             params.add(new WsExtensionParameter(CLIENT_NO_CONTEXT_TAKEOVER, null));
         }
         if (clientMaxWindowBits != -1) {
-            params.add(new WsExtensionParameter(CLIENT_MAX_WINDOW_BITS,
-                    Integer.toString(clientMaxWindowBits)));
+            params.add(new WsExtensionParameter(CLIENT_MAX_WINDOW_BITS, Integer.toString(clientMaxWindowBits)));
         }
 
         return result;
@@ -302,7 +297,7 @@ public class PerMessageDeflate implements Transformation {
 
     @Override
     public boolean validateRsvBits(int i) {
-        if ((i & RSV_BITMASK) > 0) {
+        if ((i & RSV_BITMASK) != 0) {
             return false;
         }
         if (next == null) {
@@ -314,7 +309,7 @@ public class PerMessageDeflate implements Transformation {
 
 
     @Override
-    public List<MessagePart> sendMessagePart(List<MessagePart> uncompressedParts) {
+    public List<MessagePart> sendMessagePart(List<MessagePart> uncompressedParts) throws IOException {
         List<MessagePart> allCompressedParts = new ArrayList<>();
 
         for (MessagePart uncompressedPart : uncompressedParts) {
@@ -332,8 +327,7 @@ public class PerMessageDeflate implements Transformation {
             } else {
                 List<MessagePart> compressedParts = new ArrayList<>();
                 ByteBuffer uncompressedPayload = uncompressedPart.getPayload();
-                SendHandler uncompressedIntermediateHandler =
-                        uncompressedPart.getIntermediateHandler();
+                SendHandler uncompressedIntermediateHandler = uncompressedPart.getIntermediateHandler();
 
                 deflater.setInput(uncompressedPayload.array(),
                         uncompressedPayload.arrayOffset() + uncompressedPayload.position(),
@@ -345,10 +339,14 @@ public class PerMessageDeflate implements Transformation {
                 while (deflateRequired) {
                     ByteBuffer compressedPayload = writeBuffer;
 
-                    int written = deflater.deflate(compressedPayload.array(),
-                            compressedPayload.arrayOffset() + compressedPayload.position(),
-                            compressedPayload.remaining(), flush);
-                    compressedPayload.position(compressedPayload.position() + written);
+                    try {
+                        int written = deflater.deflate(compressedPayload.array(),
+                                compressedPayload.arrayOffset() + compressedPayload.position(),
+                                compressedPayload.remaining(), flush);
+                        compressedPayload.position(compressedPayload.position() + written);
+                    } catch (NullPointerException e) {
+                        throw new IOException(sm.getString("perMessageDeflate.alreadyClosed"), e);
+                    }
 
                     if (!uncompressedPart.isFin() && compressedPayload.hasRemaining() && deflater.needsInput()) {
                         // This message part has been fully processed by the
@@ -370,55 +368,61 @@ public class PerMessageDeflate implements Transformation {
                     boolean fin = uncompressedPart.isFin();
                     boolean full = compressedPayload.limit() == compressedPayload.capacity();
                     boolean needsInput = deflater.needsInput();
+                    long blockingWriteTimeoutExpiry = uncompressedPart.getBlockingWriteTimeoutExpiry();
 
                     if (fin && !full && needsInput) {
                         // End of compressed message. Drop EOM bytes and output.
                         compressedPayload.limit(compressedPayload.limit() - EOM_BYTES.length);
-                        compressedPart = new MessagePart(true, getRsv(uncompressedPart),
-                                opCode, compressedPayload, uncompressedIntermediateHandler,
-                                uncompressedIntermediateHandler);
+                        compressedPart = new MessagePart(true, getRsv(uncompressedPart), opCode, compressedPayload,
+                                uncompressedIntermediateHandler, uncompressedIntermediateHandler,
+                                blockingWriteTimeoutExpiry);
                         deflateRequired = false;
                         startNewMessage();
                     } else if (full && !needsInput) {
                         // Write buffer full and input message not fully read.
                         // Output and start new compressed part.
-                        compressedPart = new MessagePart(false, getRsv(uncompressedPart),
-                                opCode, compressedPayload, uncompressedIntermediateHandler,
-                                uncompressedIntermediateHandler);
+                        compressedPart = new MessagePart(false, getRsv(uncompressedPart), opCode, compressedPayload,
+                                uncompressedIntermediateHandler, uncompressedIntermediateHandler,
+                                blockingWriteTimeoutExpiry);
                     } else if (!fin && full && needsInput) {
                         // Write buffer full and input message not fully read.
                         // Output and get more data.
-                        compressedPart = new MessagePart(false, getRsv(uncompressedPart),
-                                opCode, compressedPayload, uncompressedIntermediateHandler,
-                                uncompressedIntermediateHandler);
+                        compressedPart = new MessagePart(false, getRsv(uncompressedPart), opCode, compressedPayload,
+                                uncompressedIntermediateHandler, uncompressedIntermediateHandler,
+                                blockingWriteTimeoutExpiry);
                         deflateRequired = false;
                     } else if (fin && full && needsInput) {
                         // Write buffer full. Input fully read. Deflater may be
                         // in one of four states:
                         // - output complete (just happened to align with end of
-                        //   buffer
+                        // buffer
                         // - in middle of EOM bytes
                         // - about to write EOM bytes
                         // - more data to write
-                        int eomBufferWritten = deflater.deflate(EOM_BUFFER, 0, EOM_BUFFER.length, Deflater.SYNC_FLUSH);
+                        int eomBufferWritten;
+                        try {
+                            eomBufferWritten = deflater.deflate(EOM_BUFFER, 0, EOM_BUFFER.length, Deflater.SYNC_FLUSH);
+                        } catch (NullPointerException e) {
+                            throw new IOException(sm.getString("perMessageDeflate.alreadyClosed"), e);
+                        }
                         if (eomBufferWritten < EOM_BUFFER.length) {
                             // EOM has just been completed
                             compressedPayload.limit(compressedPayload.limit() - EOM_BYTES.length + eomBufferWritten);
-                            compressedPart = new MessagePart(true,
-                                    getRsv(uncompressedPart), opCode, compressedPayload,
-                                    uncompressedIntermediateHandler, uncompressedIntermediateHandler);
+                            compressedPart = new MessagePart(true, getRsv(uncompressedPart), opCode, compressedPayload,
+                                    uncompressedIntermediateHandler, uncompressedIntermediateHandler,
+                                    blockingWriteTimeoutExpiry);
                             deflateRequired = false;
                             startNewMessage();
                         } else {
                             // More data to write
                             // Copy bytes to new write buffer
                             writeBuffer.put(EOM_BUFFER, 0, eomBufferWritten);
-                            compressedPart = new MessagePart(false,
-                                    getRsv(uncompressedPart), opCode, compressedPayload,
-                                    uncompressedIntermediateHandler, uncompressedIntermediateHandler);
+                            compressedPart = new MessagePart(false, getRsv(uncompressedPart), opCode, compressedPayload,
+                                    uncompressedIntermediateHandler, uncompressedIntermediateHandler,
+                                    blockingWriteTimeoutExpiry);
                         }
                     } else {
-                        throw new IllegalStateException("Should never happen");
+                        throw new IllegalStateException(sm.getString("perMessageDeflate.invalidState"));
                     }
 
                     // Add the newly created compressed part to the set of parts
@@ -444,11 +448,15 @@ public class PerMessageDeflate implements Transformation {
     }
 
 
-    private void startNewMessage() {
+    private void startNewMessage() throws IOException {
         firstCompressedFrameWritten = false;
         emptyMessage = true;
         if (isServer && !serverContextTakeover || !isServer && !clientContextTakeover) {
-            deflater.reset();
+            try {
+                deflater.reset();
+            } catch (NullPointerException e) {
+                throw new IOException(sm.getString("perMessageDeflate.alreadyClosed"), e);
+            }
         }
     }
 
