@@ -19,14 +19,19 @@ package org.apache.tomcat.util.net;
 import java.io.File;
 import java.net.InetAddress;
 import java.net.ServerSocket;
+import java.net.SocketAddress;
+import java.nio.ByteBuffer;
+import java.nio.channels.SocketChannel;
 
 import org.junit.Assert;
+import org.junit.Assume;
 import org.junit.Test;
 
 import org.apache.catalina.connector.Connector;
 import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
 import org.apache.tomcat.jni.Pool;
+import org.apache.tomcat.util.compat.JreCompat;
 
 /**
  * Test case for the Endpoint implementations. The testing framework will ensure
@@ -196,4 +201,31 @@ public class TestXxxEndpoint extends TomcatBaseTest {
         Assert.assertNull(e);
         tomcat.getConnector().start();
     }
+
+    @Test
+    public void testUnixDomainSocket() throws Exception {
+        Tomcat tomcat = getTomcatInstance();
+        Connector c = tomcat.getConnector();
+        Assume.assumeTrue("NIO Unix domain sockets have to be supported for this test",
+                c.getProtocolHandlerClassName().contains("NioProtocol")
+                && JreCompat.isJre16Available());
+
+        File tempPath = File.createTempFile("uds-tomcat-test-", ".sock");
+        String unixDomainSocketPath = tempPath.getAbsolutePath();
+        // Need to delete the file to make way for the actual socket
+        Assert.assertTrue(tempPath.delete());
+        Assert.assertTrue(c.setProperty("unixDomainSocketPath", unixDomainSocketPath));
+        tomcat.start();
+
+        // Connect to the domain socket as a client
+        SocketAddress sa = JreCompat.getInstance().getUnixDomainSocketAddress(unixDomainSocketPath);
+        ByteBuffer response = ByteBuffer.allocate(1024);
+        try (SocketChannel socket = JreCompat.getInstance().openUnixDomainSocketChannel()) {
+            socket.connect(sa);
+            socket.write(ByteBuffer.wrap("OPTIONS * HTTP/1.0\r\n\r\n".getBytes()));
+            socket.read(response);
+        }
+
+        Assert.assertTrue((new String(response.array(), 0, response.position()).startsWith("HTTP/1.1 200")));
+    }
 }
